% script for calibrating muogenic 10Be and 26Al production parameters based on depth profile data
% based on calbhcore.m from CRONUScalc: https://bitbucket.org/cronusearth/cronus-calc
% Jakob Heyman - 2016-2017
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License, version 2,
% as published by the Free Software Foundation (www.fsf.org).

clear all;

tic();

% What version is this?
ver = '201702';

% An initial guess at the parameters.
pinit=[0.016; 142.0; 141.0; 1.9; 0.23; 12.7; 3.8];

% The parameters are:
%   p(1)             erosion rate
%   p(2)             attenuation length for 10-Be
%   p(3)             attenuation length for 26-Al
%   p(4)             fstar10 (scaled by 1.0e-3)
%   p(5)             sigma010 (scaled by 1.0e-30)
%   p(6)             fstar26 (scaled by 1.0e-3)
%   p(7)             sigma026 (scaled by 1.0e-30)

% run and load al_be constants
make_al_be_consts;
load al_be_consts;

% make consts_LSD
make_consts_LSD;
load consts_LSD;

% load Beacon Heights data. This is the UW/LLNL Beacon Heights data with uncertainty-weighted N and
% delN for samples with repeat measurements (both 10/26 - sample names marked with *). Data derived
% from http://hess.ess.washington.edu/repository/muons2016/ (some data missing in Borchers et al. 2016).
mucal = mucalib_data();
mucal.Nz10 = mucal.Nz;
mucal.Nz26 = mucal.Nz;

% set uncertainty to minimum 2.9% (10Be) or 4.9% (26Al)
unchigh10 = mucal.delN10./mucal.N10 > 0.029;
unclow10 = (unchigh10 < 1);
mucal.delN10 = mucal.delN10.*unchigh10 + mucal.N10.*unclow10.*0.029;
unchigh26 = mucal.delN26./mucal.N26 > 0.049;
unclow26 = (unchigh26 < 1);
mucal.delN26 = mucal.delN26.*unchigh26 + mucal.N26.*unclow26.*0.049;

% Get the number of samples.
nsamples10 = length(mucal.N10);
nsamples26 = length(mucal.N26);
nresids = nsamples10+nsamples26;
mucal.nresids = nresids;
% We have 7 parameters to fit.
npars=7;
mucal.npars = npars;

% Decay constants
l10 = al_be_consts.l10;
l26 = al_be_consts.l26;

% Prefs
Pref10 = al_be_consts.P10_ref_nu;
Pref26 = al_be_consts.P26_ref_nu;

% calculate atmospheric pressure
atm = ERA40atm(mucal.lat,mucal.lon,mucal.elv);

% Age Relative to t0=2010 - LSD tv from LSDfix
% tv = [0:10:50 60:100:50060 51060:1000:2000060 logspace(log10(2001060),7,200)];

% Fix w,Rc,SPhi, for sp and nu prod rate scaling 10 Ma back in time
LSDfix = LSD_fix(mucal.lat,mucal.lon,1E7,-1,consts);

% adjust to year of sampling
% time vector tv1
tv1 = LSDfix.tv;

% adjust tv, Rc, and SPhi to sampling year
if mucal.samplingyr <= 2010;
	clipidx = min(find(tv1 > 2010-mucal.samplingyr));
	tv = [2010-mucal.samplingyr tv1(clipidx:end)];
	Rc = interp1(tv1,LSDfix.Rc,tv);
	SPhi = interp1(tv1,LSDfix.SPhi,tv);
	tv = tv - 2010 + mucal.samplingyr;
else; % assume 2010 value for all years >2010
	Rc = [LSDfix.Rc(1) LSDfix.Rc];
	SPhi = [LSDfix.SPhi(1) LSDfix.SPhi];
	tv = [0 (tv1 + mucal.samplingyr - 2010)];
end;

mucal.tv = tv;

% decay factors
mucal.dcf10 = exp(-tv.*l10);
mucal.dcf26 = exp(-tv.*l26);

% surface spallation production
Psp = LSDspal(atm,Rc,SPhi,LSDfix.w,1,1,consts);
mucal.Psp10 = Psp.Be .* Pref10;
mucal.Psp26 = Psp.Al .* Pref26;

% muon production (without sigma0 and fstar!!)
% This is pre-computed and the data is included
% in mucalib_data.m to save time
%
%mucal.dz = linspace(0,8000,8001);
%Pmud = mucalib_Pmu(mucal.dz,atm,LSDfix.RcEst,consts.SPhiInf,1,1,consts);
%mucal.Pfast10d = Pmud.P_fast10';
%mucal.Pneg10d = Pmud.P_neg10';
%mucal.Pfast26d = Pmud.P_fast26';
%mucal.Pneg26d = Pmud.P_neg26';

% Use LM to find optimal values of erosion rate and attenuation length.
[pstar,iter]=mucalib_lm('mucalib_fun','mucalib_jac',pinit,1.0e-5,100,mucal);

% Compute the residual and J at the optimal parameters.
rstar=mucalib_fun(pstar,mucal);
Jstar=mucalib_jac(pstar,mucal);

% Compute Chi2 and pvalue.
chi2=norm(rstar,2)^2;
pvalue=1-chi2cdf(chi2,nresids-npars);

% Compute the covariance matrix for the fitted parameters.
covp=inv(Jstar'*Jstar);
sigmapstar=sqrt(diag(covp));

% Compute the correlation matrix for the fitted parameters.
for i=1:npars 
  for j=1:npars
    corp(i,j)=covp(i,j)/(sqrt(covp(i,i))*sqrt(covp(j,j)));
  end
end
disp('Correlations between fitted parameters');
corp

% Print out the fitted parameters.
fprintf(1,'Chi2 = %f   p-value = %f\n',[chi2; pvalue]);
fprintf(1,'Erosion Rate = %f ± %f ((g/cm2)/ka)\n',pstar(1),sigmapstar(1));
fprintf(1,'10Be Lsp = %f ± %f (g/cm2)\n',pstar(2),sigmapstar(2));
fprintf(1,'26Al Lsp = %f ± %f (g/cm2)\n',pstar(3),sigmapstar(3));
fprintf(1,'fstar10 = %f ± %f E-30\n',pstar(4),sigmapstar(4));
fprintf(1,'sigma010 = %f ± %f E-3\n',pstar(5),sigmapstar(5));
fprintf(1,'fstar26 = %f ± %f E-30\n',pstar(6),sigmapstar(6));
fprintf(1,'sigma026 = %f ± %f E-3\n',pstar(7),sigmapstar(7));


% plotting ----------------------------------------------------------
plotz = (0:10:7000);
tvzv = pstar(1).*1E-3 .* mucal.tv; % depth vector for tv (g/cm2)
tvzm = bsxfun(@plus,tvzv',plotz); % depth matrix for tv (one col per depth sample)

% sp production
Psp10rm = exp(-tvzm./pstar(2)); % spallation 10 depth prod ratio
Psp26rm = exp(-tvzm./pstar(3)); % spallation 26 depth prod ratio
Psp10m = bsxfun(@times,Psp10rm,mucal.Psp10'); % spallation 10Be production matrix
Psp26m = bsxfun(@times,Psp26rm,mucal.Psp26'); % spallation 26Al production matrix

% 10Be muon prod
Pfast10m = interp1(mucal.dz,mucal.Pfast10d,tvzm,'pchip') .* pstar(5).*1E-30; % Pfast10 surface prod
Pneg10m = interp1(mucal.dz,mucal.Pneg10d,tvzm,'pchip') .* pstar(4).*1E-3; % Pneg10 surface prod

% 26Al muon prod
Pfast26m = interp1(mucal.dz,mucal.Pfast26d,tvzm,'pchip') .* pstar(7).*1E-30; % Pfast26 surface prod
Pneg26m = interp1(mucal.dz,mucal.Pneg26d,tvzm,'pchip') .* pstar(6).*1E-3; % Pneg26 surface prod

% full production plus decay
Pfull10m = Psp10m + Pfast10m + Pneg10m; % full P10 matrix
Pfull26m = Psp26m + Pfast26m + Pneg26m; % full P26 matrix
Pfull10lm = bsxfun(@times,Pfull10m,mucal.dcf10'); % full P10 including decay matrix
Pfull26lm = bsxfun(@times,Pfull26m,mucal.dcf26'); % full P26 including decay matrix

% calculate full N by integration
N10v = trapz(mucal.tv',Pfull10lm); % calculated N10
N26v = trapz(mucal.tv',Pfull26lm); % calculated N26

% calculate sp production and N
Psp10lm = bsxfun(@times,Psp10m,mucal.dcf10'); % Psp10 including decay matrix
Psp26lm = bsxfun(@times,Psp26m,mucal.dcf26'); % Psp26 including decay matrix
Nsp10 = trapz(mucal.tv',Psp10lm); % calculated N10
Nsp26 = trapz(mucal.tv',Psp26lm); % calculated N26

% calculate mu production and N
Pfast10lm = bsxfun(@times,Pfast10m,mucal.dcf10'); % Pfast10 including decay matrix
Pfast26lm = bsxfun(@times,Pfast26m,mucal.dcf26'); % Pfast26 including decay matrix
Pneg10lm = bsxfun(@times,Pneg10m,mucal.dcf10'); % Pneg10 including decay matrix
Pneg26lm = bsxfun(@times,Pneg26m,mucal.dcf26'); % Pneg26 including decay matrix
Nfast10 = trapz(mucal.tv',Pfast10lm); % calculated N10
Nfast26 = trapz(mucal.tv',Pfast26lm); % calculated N26
Nneg10 = trapz(mucal.tv',Pneg10lm); % calculated N10
Nneg26 = trapz(mucal.tv',Pneg26lm); % calculated N26

% plot 10Be figure
figure(1);
hold on;
semilogx(Nsp10,-plotz,'r');
semilogx(Nneg10,-plotz,'g');
semilogx(Nfast10,-plotz,'b');
semilogx(N10v,-plotz,'k');
semilogx(mucal.N10,-mucal.Nz10,'.0','markersize',8);
xlabel('10Be (at/g)');
ylabel('Shielding depth (g/cm2)');
legend('spallation','slow muons','fast muons','location','south');
legend('right');
axis([1E4 1E8 -7000 0]);
hold off

% plot 26Al figure
figure(2);
hold on;
semilogx(Nsp26,-plotz,'r');
semilogx(Nneg26,-plotz,'g');
semilogx(Nfast26,-plotz,'b');
semilogx(N26v,-plotz,'k');
semilogx(mucal.N26,-mucal.Nz26,'.0','markersize',8);
xlabel('26Al (at/g)');
ylabel('Shielding depth (g/cm2)');
legend('spallation','slow muons','fast muons','location','south');
legend('right');
axis([2E4 5E8 -7000 0]);
% end plotting---------------------------------------------------
